
Hello. Setup chat connection here
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Document</title>
    <script src="https://js.pusher.com/7.0/pusher.min.js"></script>
    <link rel="stylesheet" href="/static/main.css">
</head>
<body>
    <h1>Welcome</h1>
    <form action="/flash_data" method="get">
        <p>flash messages</p>
        <p>{{ session().get("test") }}</p>
        <a href="/sessions">step 2</a>
        <input type="text" name="message" value="{{ old('message') }}">
        <button type="submit">Submit</button>
        <!-- <a href="/flash_data">Flash Data to session</a> -->
    </form>
    <script>
        const pusher = new Pusher("", {
            authEndpoint: '/broadcasting/authorize',
            cluster: "mt1"
        });

        const channel = pusher.subscribe('private-shipped');
        channel.bind('OrderProcessed', function(){
            console.log('Private Event fired')
        })
    </script>
</body>
</html>