
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Reset Password</title>
<style>
    html {
        background-color: #CBD5E1;
    }

    .button {
        color: white;
        padding-top: 7px;
        padding-bottom: 10px;
        background-color: #1F2937;
        margin-top: 15px;
    }

</style>
</head>
<body>
    <div style="width: 98%; background-color: white; margin-right: 10px; padding: 10px">
        <h4>Hello, </h4>
        <div>If you requested to reset your password, please click the button below</div>
        <a href="{{ route('change_password', {'token': token}) }}"><button class="button">Reset your password</button></a>
    </div>
</body>
</html>