""" __class__ Validation Enclosure """

from masonite.validation import RuleEnclosure


class __class__(RuleEnclosure):
    """__class__ Validation Enclosure Class."""

    def rules(self):
        """Used to return a list of rules in order to make validation
        more reusable.

        Returns:
            list -- List of rules
        """
        return [
            # Rules go here
        ]
