
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Authorizations</title>
    <link rel="stylesheet" href="/static/main.css">
</head>
<body>
    <h1>Authorizations</h1>

    {% if can("view-posts") %}
    <p>User can view posts</p>
    {% endif %}
    {% if can("display-admin") %}
    <p>User can display admin</p>
    {% endif %}
</body>
</html>