from typing import Any

class Storage:
    """File storage facade."""

    def add_driver(name: str, driver: str): ...
    def set_configuration(config: dict) -> "Storage": ...
    def get_driver(name: str = None) -> Any: ...
    def get_config_options(name: str = None) -> dict: ...
    def disk(name: str = "default") -> Any:
        """Get the file manager instance for the given disk name."""
        ...
