{{ config("application.app_url") }}
{{ asset("local", "avatar.jpg") }}
{{ url("welcome") }}
{{ optional(obj).value }}
{{ old("username") }}
{% if exists('auth/base') %}
  {{ "auth/base template exists"}}
{% endif %}

{{ method("PATCH") }}