from typing import Any

class Queue:
    def add_driver(name: str, driver: str): ...
    def get_driver(name: str) -> Any: ...
    def set_configuration(config: dict) -> "Queue": ...
    def get_config_options(driver: str) -> dict: ...
    def push(*jobs, **options) -> None: ...
    def consume(options: dict) -> Any: ...
    def retry(options: dict) -> Any: ...
